Package: seacarb
Title: Seawater Carbonate Chemistry
Version: 3.3.4
Date: 2026-04-16
LazyData: true
Authors@R: c(
  person("Jean-Pierre", "Gattuso", role = c("aut"), email = "jean-pierre.gattuso@imev-mer.fr"),
  person("James", "Orr", role = c("aut", "cre", "cph"), email = "james.orr@lsce.ipsl.fr"),
  person("Jean-Marie", "Epitalon", role = "aut", email = "jean-marie.epitalon@orange.fr"),
  person("Heloise", "Lavigne", role = "aut", email = "lavigne@obs-vlfr.fr"),
  person("Bernard", "Gentili", role = "ctb"),
  person("Mathilde", "Hagens", role = "ctb"),
  person("Andreas", "Hofmann", role = "ctb"),
  person("Jens-Daniel", "Mueller", role = "ctb"),
  person("Aurélien", "Proye", role = "ctb"),
  person("James", "Rae", role = "ctb"),
  person("Karline", "Soetaert", role = "ctb")
  )
Maintainer: James Orr <james.orr@lsce.ipsl.fr>
Depends: R (>= 3.5.0), oce, gsw, SolveSAPHE
Description: Calculates parameters of the seawater carbonate system and assists in design of ocean acidification perturbation experiments.
Encoding: UTF-8
URL: https://CRAN.R-project.org/package=seacarb
License: GPL (>= 2)
Repository: CRAN
RoxygenNote: 7.1.1
NeedsCompilation: no
Packaged: 2026-04-16 19:34:18 UTC; orr
Author: Jean-Pierre Gattuso [aut],
  James Orr [aut, cre, cph],
  Jean-Marie Epitalon [aut],
  Heloise Lavigne [aut],
  Bernard Gentili [ctb],
  Mathilde Hagens [ctb],
  Andreas Hofmann [ctb],
  Jens-Daniel Mueller [ctb],
  Aurélien Proye [ctb],
  James Rae [ctb],
  Karline Soetaert [ctb]
Date/Publication: 2026-04-17 10:10:02 UTC
Built: R 4.6.0; ; 2026-04-26 01:09:08 UTC; unix
